<?php
/**
 * Created by PhpStorm.
 * Script Name: Auth.php
 * Create: 2020/9/7 8:38
 * Description:
 * Author: fudaoji<fdj@kuryun.cn>
 */

namespace app\admin\controller;
use think\captcha\Captcha;
use think\Validate;

class Auth extends Base
{
    /**
     * @var \app\admin\model\Admin
     */
    protected $model;
    protected $needLogin = false;

    public function initialize()
    {
        parent::initialize(); // TODO: Change the autogenerated stub
        $this->model = model('admin');
    }

    /**
     * 验证码
     * @return mixed
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function verify()
    {
        $captcha = new Captcha(['fontSize' => 15]);
        return $captcha->entry(2);
    }

    /**
     * 登录
     * @return mixed
     * Author: fudaoji<fdj@kuryun.cn>
     * @throws \think\Exception
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function login(){
        if (request()->isPost()) {
            $post_data = input('post.');
            Validate::extend([
                'checkCode'=> function ($value) {
                    $captcha = new Captcha();
                    return $captcha ->check($value, 2) ? true : '验证码错误';
                }
            ]);
            $rule = [
                'verify'    => 'checkCode',
                'username'   => 'require|length:3,20',
                'password'  => 'require|length:6,20',
                '__token__' => 'require|token',
            ];
            $msg = [
                'username.require'  => '请填写账号',
                'username.length'   => '账号错误',
                'password.require' => '请填写密码',
                'password.length'  => '密码错误',
            ];
            $data = [
                'verify'   => $post_data['verify'],
                'username'   => $post_data['username'],
                'password'  => $post_data['password'],
                '__token__' => $post_data['__token__'],
            ];
            $validate = new Validate($rule, $msg);
            $result = $validate->check($data);
            if (!$result) {
                $this->error($validate->getError(), null, ['token' => request()->token()]);
                return false;
            }

            $user = $this->model->getOneByMap(['username' => $post_data['username']]);
            if ($user && $user['status'] == 1) {
                if(password_verify($post_data['password'], $user['password'])){
                    $this->model->updateOne([
                        'id' => $user['id'],
                        'ip' => request()->ip(),
                        'last_time' => time()
                    ]);
                    session($this->sKey, $user['id']);
                    if(!empty($post_data['keeplogin'])){
                        cookie('record_admin', $post_data['username']);
                    }
                    $this->success('登录成功!', cookie('redirect_url') ? cookie('redirect_url') : url('index/index'));
                }else{
                    $this->error('账号或密码错误', '', ['token' => request()->token()]);
                }
            }else{
                $this->error('用户不存在或已被禁用', '', ['token' => request()->token()]);
            }

        }

        if(session($this->sKey)){
            $this->redirect(url('index/index'));
        }
        return $this->show(['username' => cookie('record_admin')]);
    }

    /**
     * 退出
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function logout()
    {
        session(null, config("app_prefix"));
        cookie(null, config("app_prefix"));
        $this->redirect(url('login'));
    }
}